<?php

/**
 * View class for FramePress.
 *
 * This class is responsable of draw the views for controllers functions.
 * It also handle de views for controller missing errors
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link				none yet
 * @package		core
 * @subpackage	core.view
 * @since			0.1
 * @license			GPL v2 License

 * IMPORTANT NOTE: class name will be rewrited as view_[prefix] (see init.php file), to get unique class names between plugins.
 */

class PressbackupView
{

	/**
	 * List of variables to pass to the view
	 *
	 * @var array
	 * @access public
	*/
	public $vars = array();

	/**
	 * name of layout page to insert the view
	 *
	 * @var String
	 * @access public
	*/
	public $layout= 'default';

	/**
	 * Name of controller drawind the view
	 * neede to now where to find the view file
	 *
	 * @var String
	 * @access public
	*/
	public $controller_name =null;

	/**
	 * Name of controller function drawind the view
	 * neede to now where to find the view file
	 *
	 * @var String
	 * @access public
	*/
	public $function_name =null;

	/**
	 * Path for view file
	 *
	 * @var String
	 * @access public
	*/
	public $controller_file =null;

	/**
	 * Path for view file
	 *
	 * @var String
	 * @access public
	*/
	public $view_file =null;

	/**
	 * Path for layout file
	 *
	 * @var String
	 * @access public
	*/
	public $layout_file =null;


	/**
	 * Local Reference to the framework core object
	 * 
	 * @var object
	 * @access private
	 */
	private $fp = null;



	/**
	 * a reference to the Controller Obejct drawind the view
	 *
	 * @var String
	 * @access public
	*/
	private $Controller =null;

	/**
	 * Constructor.
	 *
	 * @param string $FramePress Core Class
	 * @access public
	 */
	public function __construct( &$FramePress ){
		$this->fp = $FramePress;
	}

	/**
	 * Save a variable to pass it to the view.
	 *
	 * @param string $varName tag for the variable
	 * @param mixed $value value of the variable passed to the view
	 * @return void
	 * @access public
	*/
	public function set ($varName, $value)
	{
		$this->vars[$varName] = $value;
	}

	/**
	 * Set the name of the layout that will be used for draw the view
	 *
	 * @param string $layout_name name of the layout file
	 * @return void
	 * @access public
	*/
	public function layout ($layout_name)
	{
		$this->layout = $layout_name;
	}

	/**
	*/
	public function Controller (&$controller)
	{
		$this->Controller = $controller;
	}

	
	/**
	 * Draw the view with the layout
	 *
	 * @return void
	 * @access public
	*/
	public function draw ()
	{
		$this->view_file = $this->fp->Path->Dir['VIEW'] . DS . strtolower($this->controller_name) . DS . $this->function_name . ".php";

		if(!file_exists($this->view_file)){

			$x1x_fileRelativePath = substr( $this->view_file, strpos($this->view_file, $this->fp->Path->Dir['N_ROOT']), strlen($this->view_file));
			$this->set( 'fileRelativePath', $x1x_fileRelativePath );
			$this->draw_error('missing_view'); return false;
		}

		$this->layout_file = $this->fp->Path->Dir['LAYOUT'] . DS . $this->layout . ".php";
		if(!file_exists($this->layout_file)) {
			$this->layout_file = $this->fp->Path->Dir['D_VIEW'] . DS . "default.php";
		}

		ob_start();
			//import variables
			if ($this->vars) {
				foreach ($this->vars as $key=>$value) {$$key = $value; }$this->vars = array();
			}

			//load view
			require_once ($this->view_file);

			//save all
			$content_for_layout = ob_get_contents();
		ob_end_clean();


		ob_start();
			//load layout's
			require_once ($this->layout_file);

			//save all
			$x1x_buffer = ob_get_contents();
		ob_end_clean();

		echo $x1x_buffer;
	}

	/**
	 * Draw a error view with the layout
	 *
	 * @return void
	 * @access public
	*/
	public function draw_error ($x1x_type = null, $x1x_return = false)
	{
		//import variables
		if ($this->vars) {
			foreach ($this->vars as $key=>$value) {$$key = $value; }$this->vars = array();
		}

		ob_start();
			// load view
			$x1x_file = $this->fp->Path->Dir['D_VIEW'] . DS . $x1x_type.".php";
			require_once ($x1x_file);

			//save all
			$content_for_layout = ob_get_contents();
		ob_end_clean();

		ob_start();
			//load layout's
			$x1x_file = $this->fp->Path->Dir['D_VIEW'] . DS . "default.php";
			require_once ($x1x_file);

			//save all
			$x1x_buffer = ob_get_contents();
		ob_end_clean();

		if ($x1x_return == true) { return $x1x_buffer; }

		echo $x1x_buffer;
	}


	public function __get ( $name )
	{
		if(isset($this->Controller->$name)) {
			return $this->Controller->$name;
		}

		if(isset($this->fp->$name)) {
			return $this->fp->$name;
		}
	}

}

?>
