<?php
 /**
 * Principal Controller for Pressbackup.
 *
 * This Class provide a interface to display and manage backups
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link		http://pressbackup.com
 * @package		controlers
 * @subpackage	controlers.principal
 * @since		0.1
 * @license		GPL v2 License
 */

class PressbackupMain extends PressbackupController
{
	public $layout = 'principal';

	public $Msg = null;

	public $Html = null;

	public $Session = null;

	public $Misc = null;

	public $fp = null;


	public function __construct (&$FramePress)
	{
		$this->fp = $FramePress;

		$FramePress->import('Misc.php');
		$this->Misc = new MiscPBLib();

		$FramePress->import('session.php');
		$this->Session = new PressbackupSession($FramePress);

		$FramePress->import('msg.php');
		$this->Msg = new PressbackupMsg($FramePress);
	}


	/**
	 * Redirect the user to de correct page
	 * If the user has not configured the plugin
	 * this function redirect directly to the config init page
	 *
	 * Note: this function is automaticaly called
	 * by pressing on tool menu link
	*/
	public function index()
	{
		$settings= get_option('pressbackup.preferences');

		$service = $this->Misc->currentService();

		if ($settings['configured']) {

			$this->redirect($service['dashboard']);

		} else {

			$this->redirect(array('controller' => 'settings', 'function' => 'configInit'));
		}
	}

	//------------------------------------------------------------------------------------

	/**
	 * Save the preferences to display in dash page (pressbackup's main page)
	 * Set the info to see what to see on backup tables, from, age number, etc
	 *
	 * @from string: what tab of backups see
	 * @page integer: page of backup to see
	 * @remove_schedule bool: remove all scheduled jobs befor render the page (clean start)
	 */
	public function dashOptions ($from = null, $page = null, $remove_schedule=false )
	{
		if( $page ){
			$this->Session->write('dash.page', $page);
		}

		if( $from ) {

			$saved_from = ($saved_from = $this->Session->read('dash.from'))?$saved_from:'this_site';
			$this->Session->write('dash.from', $from);

			// if tab is different of the previos saved, show the first page
			if( $saved_from != $from ){
				$this->Session->write('dash.page', 1);
			}
		}

		if( $remove_schedule ){
			$this->fp->import('Scheduler.php');
			$sch = new SchedulerPBLib();

			$sch->remove('all');
		}

		$service = $this->Misc->currentService();
		$this->redirect($service['dashboard']);
	}


	/**
	 * Renders dashboard page (the main page)
	 * This page has the interface to manage backups
	 */
	public function dashboard ()
	{
		$settings= get_option('pressbackup.preferences');

		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//check what dashboard we are seeing last time
		$from = ($saved_from = $this->Session->read('dash.from'))?$saved_from:'this_site';

		//load ajax to check backups status?
		//load on cron or on backup now status
		$reload = ( $this->Misc->getLogFile('create.log') == 'start' ||  $this->Misc->getLogFile('sent.log')  == 'start' )?'dashboard':false; 

		if($saved_reload = $this->Session->read('dash.reload')){
			$reload = $saved_reload;
			$this->Session->delete('dash.reload');
		}

		//get saved files
		if ($service['id'] != 'none' ) {

			//init
			$bucket_files= array();

			//retrive backups from Amazon S3
			if( $service['id'] == 'S3' ){
				$bucket_files = $srv->getBucket($service['bucket_name']);
			}

			//retrive backups from Pro
			elseif( $service['id'] == 'Pro' || $service['id'] == 'Exp' ){
				$bucket_files = $srv->getFilesList();
			}

			//retrive backups from DropBox
			elseif( $service['id'] == 'dBox' ){
				$bucket_files = $srv->getFilesList();
			}

			//retrive backups from Local
			elseif( $service['id'] == 'Local' ){
				$bucket_files = $srv->getFilesList();
			}

			if (!$bucket_files) {
				$this->set('bucket_files', array());
			} else {

				//get a normalized list and sorted by date
				$bucket_files = @$this->Misc->msort($service['id'], $bucket_files);
				$bucket_files = $this->Misc->filterFiles ($from, $bucket_files);

				$pagination_size = 5;
				$page = ($page_saved = $this->Session->read('dash.page'))?$page_saved:1;

				//fix page value when it is incorrect ( a page that not exists, like page > pages )
				if(!array_slice($bucket_files, (($page -1 )*$pagination_size), $pagination_size, true) && $page > 1){
					$page--;
					$this->Session->write('dash.page', $page);
				}

				//build paginator
				$paginator['page'] = $page ;
				$paginator['total']  = count($bucket_files);
				$paginator['pages'] = ceil ($paginator['total'] /$pagination_size);
				$paginator['ini']  = (($page -1 )*$pagination_size);
				$paginator['fin']  = (($page*$pagination_size) -1);
				$paginator['func_path']  = array('controller'=>'main', 'function'=>'dashOptions', $from);
				$paginator['pagination'] = $pagination_size;

				//set paginator to view
				$this->set('paginator', $paginator);
				$this->set('bucket_files', array_slice($bucket_files, $paginator['ini'], $pagination_size, true));
			}

			//to show on dashboard when the next backup happen
			$this->set('next_scheduled_job', wp_next_scheduled('pressbackup.cron.doBackupAndSaveIt'));

			//inform about the first time backup
			if(get_option('pressbackup.firstRun') == true ) {
				$this->Msg->set('info', __('We are processing a first backup, if you do not see a progress bar you can refresh the page', 'pressbackup'));
			}
		}

		//remove first time glag
		if(get_option('pressbackup.firstRun') == true ) {
			update_option('pressbackup.firstRun', false);
		}

		//set messages
		if($this->Session->check('general_msg') || $this->Session->check('general_msg', true)){
			$this->Msg->set('info', $this->Session->read('general_msg'));
			$this->Msg->set('info', $this->Session->read('general_msg', true));
			$this->Session->delete('general_msg');
			$this->Session->delete('general_msg', true);
		}

		//set errors
		if($this->Session->check('error_msg') || $this->Session->check('error_msg', true)){
			$this->Msg->set('error', $this->Session->read('error_msg'));
			$this->Msg->set('error', $this->Session->read('error_msg', true));
			$this->Session->delete('error_msg');
			$this->Session->delete('error_msg', true);
		}

		$this->set('settings', $settings);
		$this->set('from', $from);
		$this->set('service', $service);
		$this->set('reload', $reload);
		$this->set('timezone_string', $this->Misc->timezoneString());
	}

	/**
	 * Shows dashboardExpress page (the main page)
	 * This page has the interface to manage backups
	 *
	 * @from string: what tab of backups see
	 * @reload bool: load ajax checker
	 * @page integer: page of backup to see
	 */
	public function dashboardExpress ()
	{
		$this->redirect($service['dashboard']);
	}

	/**
	 * Shows upload form for restore a backup
	 */
	public function uploadAndRestoreBackup ()
	{
		$settings= get_option('pressbackup.preferences');

		//set error from previus upload
		if($this->Session->check('error_msg')){
			$this->Msg->set('error', $this->Session->read('error_msg'));
			$this->Session->delete('error_msg');
		}

		//check for folder permissions
		$permissions = array();
		$disable = false;
		$msg = false;

		$dir = WP_CONTENT_DIR . DS;
		if( !@is_writable($dir) ) { $permissions[] ='wp_content'; }
		if( !@is_writable($dir . 'themes' . DS ) ) { $permissions[] ='themes'; }
		if( !@is_writable($dir . 'plugins' . DS ) ) { $permissions[] ='plugins'; }
		if( @file_exists($dir . 'uploads' ) && !@is_writable($dir . 'uploads' . DS ) ) { $permissions[] ='uploads'; }


		if($permissions) {
			$denied_perms="<b>".join('</b>, <b>', $permissions)."</b>";
			$msg = sprintf(__('Permissions denied to write on %1$s folder/s','pressbackup'),$denied_perms);
			$msg .= "<br/> ";
			$msg .= __('Pleace change the permissions of those folders to 777','pressbackup');
			$this->Msg->set('error', $msg);
			$disable = true;
		}

		$this->set('upload_size', $this->Misc->getByteSize($this->Misc->uploadMaxFilesize(),'mb'));
		$this->set('settings', $settings);
		$this->set('disable_ulpoad', $disable);
	}

	//------------------------------------------------------------------------------------

	/**
	 * Start a manual backup
	 * create a schedule action (background proccess)
	 *
	 * @reloadPage string: specify what type of manual backup we are doing
	 * @to_include string: ignore stored setings and backup what this var says
	 */
	public function backupStart ($reloadPage ="backupDownload", $to_include=null)
	{
		$settings = get_option('pressbackup.preferences');

		$this->fp->import('SimpleCurl.php');
		$curl = new SimpleCurlPBLib();

		$this->fp->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		$sch->remove('pressbackup.main.doBackupAndSaveIt');
		$sch->remove('pressbackup.main.doBackupAndDownloadIt');

		//clean log && tmp dir
		$this->Misc->cleanTmpFordes();

		if($to_include){
			$this->Session->write('type_temp',$to_include,true);
		}

		$auxCompBack = $settings['compatibility']['background'];

		//background creation type: SOFT
		if ($auxCompBack == 10 && $reloadPage == "backupDownload") {
			$sch->add('+4 seconds', 'pressbackup.main.doBackupAndDownloadIt');
		}

		elseif ($auxCompBack == 10 && $reloadPage == "dashboard") {
			$sch->add('+4 seconds', 'pressbackup.main.doBackupAndSaveIt');
		}

		//background creation type: MIDIUM
		if ($auxCompBack == 20 && $reloadPage == "backupDownload") {
			$curl->call(array(
				'url' => get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup.ajax.doBackupAndDownloadIt',
				'cookie' => $_COOKIE,
				'timeout' => 4,
			));
		}

		elseif ($auxCompBack == 20 && $reloadPage == "dashboard") {
			$curl->call(array(
				'url' => get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup.ajax.doBackupAndSaveIt',
				'cookie' => $_COOKIE,
				'timeout' => 4,
			));
		}

		//background creation type: HARD
		elseif ($auxCompBack == 30) {

			if ($reloadPage == "backupDownload") {
				$action = get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup.ajax.doBackupAndDownloadIt';
			}
			elseif ($reloadPage == "dashboard") {
				$action = get_bloginfo('wpurl').'/wp-admin/admin-ajax.php?action=pressbackup.ajax.doBackupAndSaveIt';
			}

			$cookie = array(); foreach($_COOKIE as $key => $value) { $cookie[]=$key.'='.$value;}

			$info = array();
			$info[0] = $action;
			$info[1]= join (';', $cookie);

			$args = array(
				'file' => $this->fp->Path->Dir['LIB'].DS.'background.php',
				'split'=>'ALLYOUNEEDISLOVE',
				'args'=>base64_encode(join('|ALLYOUNEEDISLOVE|', $info)),
			);
			$this->Misc->php($args);
		}

		$this->Session->write('dash.reload', $reloadPage);
		$service = $this->Misc->currentService();
		$this->redirect($service['dashboard']);
	}

	/**
	 * Create and send a backup to a storage service (S3, PPro, etc )
	 * this is the background proccess called from
	 * backup start or via scheduled task (cron job)
	 */
	public function createBackupAndSaveIt()
	{
		//try to cut the conection
		@ignore_user_abort(true);
		@set_time_limit(0);
		@ob_end_clean();
		@ob_start();
		header("Status: 204", true);
		header("Content-type: text/html", true);
		header("Content-Length: 0", true);
		header("Connection: close", true);
		@ob_end_flush();
		@ob_flush();
		@flush();
		if(defined('STDIN')){ @fclose(STDIN); }
		if(defined('STDOUT')){ @fclose(STDOUT); }
		if(defined('STDERR')){ @fclose(STDERR); }

		//tools

		$settings= get_option('pressbackup.preferences');

		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		$this->fp->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		$sch->remove('pressbackup.main.doBackupAndSaveIt');

		$is_cron = !$this->Session->check('type_temp',true);

		//set backup type on "cron" && advanced backups
		if ( $is_cron ){
			$tasksDone = $settings['backup']['type'] = $sch->tasksToRunNow($settings);
			update_option('pressbackup.preferences', $settings);
		}

		//set backup type on "backup now" 
		if(!$is_cron){
			$aux_type = $sch->activatedTasks($settings);
			$settings['backup']['type'] = $this->Session->read('type_temp',true);
			update_option('pressbackup.preferences', $settings);
		}

		//make and save backup
		set_error_handler(array($this, 'creation_eh'));

		if($file = $bm->create()) {
			$bm->save($file);
		}

		restore_error_handler();

		//delete files from tmp folder
		$this->Misc->perpareFolder($this->fp->Path->Dir['PBKTMP']);

		//restore activated types on backup now 
		if(!$is_cron){
			$settings['backup']['type'] = $aux_type;
			$this->Session->delete('type_temp',true);
			update_option('pressbackup.preferences', $settings);
		}

		if($is_cron){
			//restore activated types
			$settings['backup']['type'] = $sch->activatedTasks($settings);
			update_option('pressbackup.preferences', $settings); 
			$sch->updateTaskLastDate($tasksDone);
		}

		//renew cron
		$sch->add();

		exit;
	}

	/**
	 * Create a backup to downloading it latter
	 * this is the background proccess called from
	 * backup start. The ajax checker tell de browser when
	 * this procces finish and redirect the user to donwload backup
	 */
	public function createBackupAndDownloadIt ()
	{
		@set_time_limit(0);

		$settings= get_option('pressbackup.preferences');

		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		$this->fp->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		$sch->remove('pressbackup.main.doBackupAndDownloadIt');

		//set backup type on "backup now" otherwise it will backup the predefined settings
		if($this->Session->check('type_temp',true)){
			$aux_type = $settings['backup']['type'];
			$settings['backup']['type'] = $this->Session->read('type_temp',true);
			update_option('pressbackup.preferences', $settings);
		}

		//create the file
		set_error_handler(array($this, 'creation_eh'));
		$zip_file_created = $bm->create();
		restore_error_handler();

		if($zip_file_created){
			$this->Misc->writeLogFile('create.return', $zip_file_created);
		}

		//restore activated types on backup now 
		if($this->Session->check('type_temp',true)){
			$settings['backup']['type'] = $aux_type;
			$this->Session->delete('type_temp',true);
			update_option('pressbackup.preferences', $settings);
		}

		exit;
	}

	//------------------------------------------------------------------------------------

	/**
	 * Call to restore if uploaded backups its valid
	 * or save the errors and display upload form again
	 */
	public function backupRestoreFromUpload ()
	{
		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		//return if exists upload errors
		if(!$this->Misc->checkBackupUpload() || !$this->Misc->checkBackupIntegrity($_FILES['backup']['tmp_name']))
		{
			$this->redirect(array('controller'=>'main', 'function'=>'uploadAndRestoreBackup'));
		}

		//restore from uploaded file
		$bm->restore(array('tmp_name'=>$_FILES['backup']['tmp_name']));

		//delete files from tmp folder
		$this->Misc->perpareFolder($this->fp->Path->Dir['PBKTMP']);

		$service = $this->Misc->currentService();
		$this->redirect($service['dashboard']);
	}

	/**
	 * Restore a backup previously stored by backup_upload
	 * or  S3/PPro get function ( from backup function lib)
	 */
	function backupRestoreFromService ($name = null)
	{
		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		//get current service
		$service = $this->Misc->currentService();

		//check for folder permissions on wp-content folder and subfollders
		$permissions = array ();
		$dir = WP_CONTENT_DIR . DS;

		if( !@is_writable($dir) ) { $permissions[] ='wp_content'; }
		if( !@is_writable($dir . 'themes' . DS ) ) { $permissions[] ='themes'; }
		if( !@is_writable($dir . 'plugins' . DS ) ) { $permissions[] ='plugins'; }
		if( @file_exists($dir . 'uploads' ) && !@is_writable($dir . 'uploads' . DS ) ) { $permissions[] ='uploads'; }

		if($permissions) {
			$denied_perms="<b>".join('</b>, <b>', $permissions)."</b>";
			$msg = sprintf(__('Permissions denied to write on %1$s folder/s','pressbackup'),$denied_perms);
			$msg .= "<br/> ";
			$msg .= __("Please, change the permissions of those folders to 777 if you want to be able to restore a backup",'pressbackup');
			$this->Session->write('error_msg', $msg);
			$this->redirect($service['dashboard']); exit;
		}

		//get the file and store it on a tmp folder
		$tmp_name = $bm->get($name);

		//go back on bad file
		if(!$tmp_name || !$this->Misc->checkBackupIntegrity($tmp_name)){
			$this->redirect($service['dashboard']); exit;
		}

		//r-r-r-restore it
		$bm->restore(array('tmp_name'=>$tmp_name));

		//delete files from tmp folder
		$this->Misc->perpareFolder($this->fp->Path->Dir['PBKTMP']);

		$this->redirect($service['dashboard']); exit;
	}

	/**
	 * Download a backup From S3
	 *
	 * Download a backup previously stored by
	 * S3 get function ( from backup function lib)
	 */
	function backupGet ($name=null)
	{
		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		//get current service
		$service = $this->Misc->currentService();

		//Exclusive action for pressbackup memberships
		if (in_array($service['id'], array('Exp', 'Pro'))){

			//make it available on our servers
			$resource = $bm->get2($name);

			//go back on bad file
			if(!$resource){
				$this->redirect($service['dashboard']); exit;
			}

			//download it from our servers
			$this->redirect($resource['download']); exit;
		}

		//get the file and store it on a tmp folder
		$tmp_name = $bm->get($name);

		//go back on bad file
		if(!$tmp_name || !$this->Misc->checkBackupIntegrity($tmp_name)){
			$this->redirect($service['dashboard']); exit;
		}

		$this->redirect(array('function' => 'backupDownload', base64_encode($tmp_name)));
	}


	/**
	 * Download a backup
	 * the backup was presviously stored by createBackupAndDownloadIt
	 */
	public function backupDownload ($file =null)
	{
		//clean output buffer (where echos and prints are)
		@ob_end_clean();

		$file = base64_decode($file);

		//fix for no occidental characers, basename and pathinfo remove them from $file
		$pathinfo = explode(DS, $file);
		$name = array_pop($pathinfo);

		//set clean layout and set the file name to the view
		$this->layout = 'blank';
		$this->set('file', $file);
		$this->set('name', $name);

		//clean all tmp folder after download
		register_shutdown_function (array($this->Misc, 'cleanTmpFordes'));
	}


	/**
	 * Remove a backup From PPro/S3
	 *
	 * @name string: name of the backup to remove
	 */
	public function backupDelete ($name=null)
	{
		$this->fp->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		//Delete the file
		$bm->delete($name);

		//redirect to the correct dashboard
		$service = $this->Misc->currentService();
		$this->redirect($service['dashboard']);
	}


	//------------------------------------------------------------------------------------


	/**
	 * Check the status of the backup now process
	 *
	 * This function its called via Ajax and inform what
	 * is the process doing. And most important when
	 * the process finish.
	 */
	public function cronBackupStatus ()
	{
		//get background process status
		$response = $this->Misc->backgroundStatus();

		//clean output buffer
		ob_end_clean();

		//send json
		header("Status: 200", true);
		header("Content-type: application/json", true);
		echo $response;
		exit;
	}



	/**
	 * Background process error handler
	 *
	 * This function register the errors ocurred on the background process
	 */
	function creation_eh($level, $message, $file, $line, $context)
	{
		$this->Misc->writeLogFile('create.fail', $this->Misc->getLogFile('create.fail') . "<br> \n" .$message );
		return false;
	}


	//------------------------------------------------------------------------------------

	/**
	 * Shows host information && background status (for report an error)
	 */
	public function reportInfo( $resetScheduler = false, $cleanTMPFolders = false, $cleanErrorLog = false)
	{
		//basic settings info
		$service = $this->Misc->currentService();
		$plugin_info = get_plugin_data($this->fp->main_file);
		$basicInfo = array(
			'Host'=> array(
				'modules' =>  get_loaded_extensions(),
				'type' => $_SERVER['SERVER_SOFTWARE'],
				'sapi' => php_sapi_name(),
				'port' => $_SERVER['SERVER_PORT'],
				'mem_max' => ini_get('memory_limit'),
				'mem_used' => memory_get_peak_usage(true),
				'tmp_dir' => $this->fp->Path->Dir['SYSTMP'],
				'tmp_free' =>  disk_free_space($this->fp->Path->Dir['SYSTMP']),
			),
			'User' => array(
				'browser' => $_SERVER['HTTP_USER_AGENT']
			),
			'WP'=> array(
				'version' => get_bloginfo ('version'),
				'url' => get_bloginfo ('wpurl'),
			),
			'Plugin'=> array(
				'version' => $plugin_info['Version'],
				'service' => $service['id'],
			),
		);

		$logs = array(
			'errors' => $this->Misc->getLogFile('create.fail'),
			'creation' => $this->Misc->getLogFile('create.log'),
			'save' => $this->Misc->getLogFile('sent.log'),
		);

		//folder contents
		$tmp_dir_path = $this->fp->Path->Dir['PBKTMP'] . DS;
		$files = array();
		if(is_dir($tmp_dir_path)) {

			$tmp_dir = scandir($tmp_dir_path);
			for($i=0; $i < count($tmp_dir); $i++){

				if(in_array($tmp_dir[$i], array('.', '..'))){continue;}

				$files[]=array(
					'name' => $tmp_dir[$i],
					'size' => filesize ($tmp_dir_path.DS.$tmp_dir[$i]),
					'download'=> base64_encode($tmp_dir_path.DS.$tmp_dir[$i]),
				);
			}
		}

		if ($resetScheduler) {
			$this->fp->import('Scheduler.php');
			$sch = new SchedulerPBLib();
			$sch->remove('all');
		}

		if ($cleanTMPFolders) {
			$this->Misc->cleanTmpFordes();
		}

		if ($cleanErrorLog) {
			$this->Misc->deleteLogFile('create.fail');
		}

		$this->set('info', $basicInfo);
		$this->set('logs', $logs);
		$this->set('files', $files);
	}

}
?>
