<?php

/**
 * Settings Controller for Pressbackup.
 *
 * This Class provide a interface to display and manage Plugin settings
 * Also Manage the configurattion wizard
 *
 * Licensed under The GPL v2 License
 * Redistributions of files must retain the above copyright notice.
 *
 * @link			http://pressbackup.com
 * @package			controlers
 * @subpackage		controlers.settings
 * @since			0.1
 * @license			GPL v2 License
 */
 
class PressbackupSettings extends PressbackupController
{
	public $layout = 'principal';

	public $Msg = null;

	public $Html = null;

	public $Session = null;

	public $Misc = null;

	public $fp = null;

	public function __construct (&$FramePress)
	{
		$this->fp = $FramePress;

		$FramePress->import('Misc.php');
		$this->Misc = new MiscPBLib();

		$FramePress->import('session.php');
		$this->Session = new PressbackupSession($FramePress);

		$FramePress->import('msg.php');
		$this->Msg = new PressbackupMsg($FramePress);
	}

	/**
	 * Redirect the user to de correct page
	 * If the user has not configured the plugin
	 * this function redirect directly to the config init page
	 *
	 * Note: this function is automaticaly called
	 * by pressing on settings menu link
	 */
	public function index()
	{
		$settings = get_option('pressbackup.preferences');

		if ($settings['configured']) {
			$this->redirect(array('controller' => 'settings', 'function' => 'configCredentials'));
		} else {
			$this->redirect(array('controller' => 'settings', 'function' => 'configInit'));
		}
	}

	/**
	 * Start The configuration wizard
	 * Check all that all nedded libs are intalled
	 * Also check for perms on tmp folders
	 */
	public function configInit()
	{
		$settings = get_option('pressbackup.preferences');

		$error = array();
		$disable_backup_files = false;
		$need_to_update_preferences = false;

		//check for
		if (strpos($_SERVER['SERVER_SOFTWARE'], 'iis') !== false) {
			$error['iss'] = __('Sorry your web Server (Windows IIS) is not complatible with Pressbackup. You are welcome to use Pressbackup, but we cannot provide any support or guarantees for your system','pressbackup');
		}

		//check for
		if (ini_get('safe_mode')) {
			$error['safe_mode'] = __('Safe mode is enabled. You would need to contact your hosting provider and ask them if it\'s  necessary for you to have safe_mode enabled. Most hosts should be able  to disable it for you, or give you a more specific answer','pressbackup');
			$disable_backup_files = true;
		}

		//check for Zip Creation
		$bin = $this->Misc->checkShell('zip');
		if (!$bin) {
			$settings['compatibility']['zip'] = 10;
			$need_to_update_preferences = true;
		}
		if (!$bin && !class_exists('ZipArchive')) {
			$error['zip'] = __('You probably don\'t have the php-zip extension installed. You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		$blog_url = get_bloginfo( 'wpurl' );

		//Check for SSL
		$parsed_url = parse_url($blog_url);
		if($parsed_url['scheme'] == 'https'){
			$settings['compatibility']['background'] = 20;
			$need_to_update_preferences = true;
		}

		// Check for CURL
		if (!extension_loaded('curl')) {
			$error['curl'] = __('You probably don\'t have the php-curl extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		// Check CURL local
		$this->import('SimpleCurl.php');
		$curl = new SimpleCurlPBLib();

		$curl->call(array(
			'url' => $blog_url,
			'parse.response' => false
		));

		if(!in_array($curl->response['code'], array(200, 301))) {
			$error['curl'] = __('You probably don\'t have permission to run curl locally.  You would need to contact your hosting provider and ask them if loopback HTTP requests are enabled.','pressbackup');
			$disable_backup_files = true;
		}

		//check for
		if (!class_exists('SimpleXMLElement')) {
			$error['xml'] = __('You probably don\'t have the SimpleXML extension installed.  You would need to contact your hosting provider and ask them to install it.','pressbackup');
			$disable_backup_files = true;
		}

		//tmp dir
		if (!file_exists($this->Path->Dir['PBKTMP']) && !mkdir($this->Path->Dir['PBKTMP'])) {
			$error['tmpdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $this->Path->Dir['PBKTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($this->Path->Dir['PBKTMP'], 0777);
		}

		//log dir
		if (!file_exists($this->Path->Dir['LOGTMP']) && !mkdir($this->Path->Dir['LOGTMP'])) {
			$error['logdir'] = sprintf(__('Could not create the FileStore directory "%1$s". Please check the effective permissions.','pressbackup'), $this->Path->Dir['LOGTMP']);
			$disable_backup_files = true;
		} else {
			@chmod($this->Path->Dir['LOGTMP'], 0777);
		}

		//validate use Cron
		$checkCron = get_option('pressbackup.wizard.cronTaskStatus');
		if ($checkCron == "fail") {
			$error['cron'] = __('Could not create a Cron Job Backup. Try it Again refreshing the page, or contact Pressbackup Support.', 'pressbackup');
			$disable_backup_files = true;
			update_option('pressbackup.wizard.cronTaskStatus', 'no checked');
		}
		elseif (!$disable_backup_files && $checkCron  != "success" ){
			$this->import('Scheduler.php');
			$sch = new SchedulerPBLib();
			$sch->add('+4 seconds', 'pressbackup.wizard.doCronTask');
			$this->set('check_cron', true);
		}

		//update preferences if its require
		if($need_to_update_preferences){
			update_option('pressbackup.preferences', $settings);
		}

		//set errors
		if($this->Session->check('error') ){
			$error['auth'] = $this->Session->read('error');
			$this->Session->delete('error');
		}

		$this->set('error', $error);
		$this->set('disable_backup_files', $disable_backup_files);
	}

	/*
	 * 
	 */
	public function configDecide()
	{
		if($_POST['data']['selected_product'] == 'free'){

			$this->redirect(array('controller' => 'settings', 'function' => 'configStep1'));

		}elseif ($_POST['data']['selected_product'] == 'pay'){

			$this->redirect(array('controller' => 'settings', 'function' => 'configmembershipStep1Save'));
		}
	}

	/**
	 * Show the form to select the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	public function configStep1()
	{
		$service = false;
		if ($this->Session->check('error_msg')) {

			//save selected credentials && set error msg
			$service = $this->Session->read('service');
			$this->Msg->set('error',$this->Session->read('error_msg'));

			$this->Session->delete('error');
			$this->Session->delete('service');
			$this->Session->delete('error_msg');
		}

		$this->set('service', $service);
		$this->set('settings', get_option('pressbackup.preferences'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	public function configStep1Save()
	{
		//Exit if wrong parameters
		if (!isset($_POST['data']) || !$this->checkFieldsStep1($_POST['data'])) {
			$this->Session->write('error', true);
			$this->Session->write('service', $_POST['data']['service']);
			$this->redirect(array('controller' => 'settings', 'function' => 'configStep1'));
		}

		$settings = get_option('pressbackup.preferences');

		$settings['Service']['s3']['credential'] = '';
		$settings['Service']['pressbackuppro']['credential'] = '';
		$settings['Service']['pressexpress']['credential'] = '';
		$settings['Service']['dropbox']['credential'] = '';
		$settings['Service']['local']['path'] = '';


		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$srv = $this->Misc->serviceObject('S3', array($PS3['accessKey'], $PS3['secretKey']));

			//check if bucket exists
			$buckets = @$srv->listBuckets();
			$the_bucket = $this->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;
			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $this->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$srv->putBucket($the_bucket, ServiceS3PBLib::ACL_PRIVATE, $region)) {
				$this->Session->write('error', true);
				$this->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				$this->Session->write('service', $_POST['data']['service']);
				$this->redirect(array('controller' => 'settings', 'function' => 'configStep1'));
			}

			//values to save
			$settings['Service']['s3']['bucket_name'] = $the_bucket;
			$settings['Service']['s3']['region'] = $region;
			$settings['Service']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];
			$settings['Service']['local']['path'] = $PLOCAL['path'];

		}

		update_option('pressbackup.preferences', $settings);


		if ($_POST['data']['service'] == 'dbox') {

			$srv = $this->Misc->serviceObject('dBox', array(null, null));
			$srv->auth(array('menu_type'=>'menu', 'controller' => 'settings', 'function' => 'dropboxAuthorize'));

		} else {

			$this->redirect(array( 'controller' => 'settings', 'function' => 'configStep2'));
		}
	}


	/**
	 * Show the form to enter prefered settings
	 * as backup type, schedule time, etc
	 */
	public function configStep2()
	{
		$settings = get_option('pressbackup.preferences');

		//check if there are errors to show
		if ($this->Session->check('error')) {
			$this->Msg->set('error',$this->Session->read('error_msg'));
			$this->Session->delete('error');
			$this->Session->delete('error_msg');
		}

		$this->set('settings', $settings);
		$this->set('service', $this->Misc->currentService());
	}

	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	public function configStep2Save()
	{
		$settings = get_option('pressbackup.preferences');

		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->checkFieldsStep2($_POST['data']['preferences'])) {
			$this->Session->write('error', true);
			$this->redirect(array('controller' => 'settings', 'function' => 'configStep2'));
		}

		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$settings['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$settings['backup']['copies'] = 5;
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		// saving backup schedule time
		if ($advanced_activated) {
			$minTimeTask = $sch->minScheduledTaskTime($_POST['data']['preferences']);
			$settings['backup']['time'] = $minTimeTask['time'];
		}
		elseif ( isset ($_POST['data']['preferences']['time']) ) {
			$settings['backup']['time'] = $_POST['data']['preferences']['time'];
		}
		else {
			$settings['backup']['time'] = 24;
		}

		//setting backup type
		if($advanced_activated){

			$curdate = strtotime($this->Misc->midnightUTC());
			$post = $_POST['data']['preferences']['backup_advanced'];

			$settings['backup_advanced']['activated'] = true;
			$settings['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$settings['backup']['type'] = $sch->activatedTasks($settings['backup_advanced']);
		}else{
			$settings['backup_advanced']['activated'] = false;
			$settings['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		$settings['configured'] = 1;

		update_option('pressbackup.preferences', $settings);

		$service = $this->Misc->currentService();
		if ($service['id'] != 'none') {
			//create a first new
			$sch->add('+1 seconds');
			//ignore settings and make a first backup with all
			$this->Session->write('type_temp', '7,5,3,1',true);
			//tell dash to load ajax to chk backup status
			$this->Session->write('dash.reload', 'dashboard');
		}

		$this->redirect(array('controller' => 'main', 'function' => 'dashboard'));
	}

	/**
	 * Store the credentials if they are correct
	 * or save the error and return to form page
	 */
	public function configmembershipStep1Save()
	{
		$settings = get_option('pressbackup.preferences');

		$srv = $this->Misc->serviceObject('Pro', array(null, null));

		if(!$res = $srv->auth()) {
			$this->Session->write('error', __('Please register your site in https://pressbackup.com', 'pressback'));
			$this->redirect(array('controller'=>'settings', 'function'=>'configInit')); exit;
		}

		$str = base64_encode($res['username'] . '|AllYouNeedIsLove|' . $res['authkey']);

		if($res['package'] == 'pro'){
			$settings['Service']['pressbackuppro']['credential'] = $str;
		}else{
			$settings['Service']['pressexpress']['credential'] = $str;
		}

		update_option('pressbackup.preferences', $settings);

		$this->configmembershipStep2Save();
	}

	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 */
	public function configmembershipStep2Save()
	{
		$settings = get_option('pressbackup.preferences');

		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		/**
		 * Config Default Pressbackup Express
		 */
		$settings['backup']['copies'] = 7;
		$settings['backup']['time'] = 24;
		$settings['backup']['type'] = '7,5,3,1';

		$settings['configured'] = 1;

		update_option('pressbackup.preferences', $settings);

		$sch->add('+1 seconds');sleep(1);


		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject('Pro');
		$srv->pingConfigured();

		$this->Session->write('dash.reload', 'dashboard');
		$this->redirect($service['dashboard']);
	}


	/**
	 * Show the form to modify the service:
	 * S3, Pressbackup Pro, localhost, Dropbox, etc
	 */
	public function configCredentials()
	{
		$cs = $this->Misc->currentService();
		$service = $cs['id'];

		if ($this->Session->check('error')) {

			$this->Msg->set('error',$this->Session->read('error_msg'));
			$service = $this->Session->read('service');

			$this->Session->delete('error');
			$this->Session->delete('service');
			$this->Session->delete('error_msg');
		}

		$this->set('service', $service);
		$this->set('services', $this->Misc->service());
		$this->set('settings', get_option('pressbackup.preferences'));

	}

	/**
	 * Store the modified credentials if they are correct
	 * or save the error and return to form page
	 */
	public function configCredentialsSave()
	{
		if (!isset($_POST['data']) || !$this->checkFieldsStep1($_POST['data'])) {
			$this->Session->write('error', true);
			$this->Session->write('service', $_POST['data']['service']);
			$this->redirect(array('controller' => 'settings', 'function' => 'configCredentials'));
		}

		$settings = get_option('pressbackup.preferences');

		$this->import('BackupsManager.php');
		$bm = new BackupsManagerPBLib();

		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();

		$settings['Service']['s3']['credential'] = '';
		$settings['Service']['pressbackuppro']['credential'] = '';
		$settings['Service']['pressexpress']['credential'] = '';
		$settings['Service']['dropbox']['credential'] = '';
		$settings['Service']['local']['path'] = '';

		if ($_POST['data']['service'] == 's3') {

			$PS3 = $_POST['data']['preferences']['S3'];

			$srv = $this->Misc->serviceObject('S3', array($PS3['accessKey'], $PS3['secretKey']));

			//check if bucket exists
			$buckets = @$srv->listBuckets();
			$the_bucket = $this->Config->read('S3.bucketname') . '-' . md5(uniqid(rand(), true));
			$create_bucket = true;

			for ($i = 0; $i < count($buckets); $i++) {
				if (strpos($buckets[$i], $this->Config->read('S3.bucketname')) !== false) {
					$the_bucket = $buckets[$i];
					$create_bucket = false;
					break;
				}
			}

			//create bucket if it not exist
			$region = ($PS3['region'] == 'EU') ? 'EU' : false;
			if ($create_bucket && !$srv->putBucket($the_bucket, ServiceS3PBLib::ACL_PRIVATE, $region)) {
				$this->Session->write('error', true);
				$this->Session->write('error_msg', __("Unable to create a bucket on S3: Service temporarily unavailable. Please try it again later.",'pressbackup'));
				$this->Session->write('service', $_POST['data']['service']);
				$this->redirect(array('controller' => 'settings', 'function' => 'configCredentials'));
			}

			//values to save
			$settings['Service']['s3']['bucket_name'] = $the_bucket;
			$settings['Service']['s3']['region'] = $region;
			$settings['Service']['s3']['credential'] = base64_encode($PS3['accessKey'] . '|AllYouNeedIsLove|' . $PS3['secretKey']);

		} elseif ($_POST['data']['service'] == 'pro') {

			$PPRO = $_POST['data']['preferences']['pressbackuppro'];
			$settings['Service']['pressbackuppro']['credential'] = base64_encode($PPRO['user'] . '|AllYouNeedIsLove|' . $PPRO['key']);

		} elseif ($_POST['data']['service'] == 'express') {

			$PEXP = $_POST['data']['preferences']['pressexpress'];
			$settings['Service']['pressexpress']['credential'] = base64_encode($PEXP['user'] . '|AllYouNeedIsLove|' . $PEXP['key']);
			//set default values for express
			$settings['backup_advanced']['activated'] = false;
			$settings['backup']['copies'] = 7;
			$settings['backup']['time'] = 24;
			$settings['backup']['type'] = '7,5,3,1';

		} elseif ($_POST['data']['service'] == 'local') {

			$PLOCAL = $_POST['data']['preferences']['local'];
			$settings['Service']['local']['path'] = $PLOCAL['path'];
		} 


		if ( $_POST['data']['service'] == 'no') {
			$settings['backup_advanced']['activated'] = false;
		}

		update_option('pressbackup.preferences', $settings);


		$service = $this->Misc->currentService();

		//remove schedule if sleected none, o add it id selected other option
		if ($service['id'] != 'none') { $sch->add(); } else { $sch->remove(); }

		//inform about this action to pressbackup
		if ( in_array( $service['id'], array('Pro', 'Exp') )) {

			$srv = $this->Misc->serviceObject('Pro');
			$srv->pingConfigured();
		}

		//redirect to dropbox if taht option was selected 
		if ($_POST['data']['service'] == 'dbox') {
			$srv = $this->Misc->serviceObject('dBox', array(null, null));
			$srv->auth(array('menu_type'=>'menu', 'controller' => 'settings', 'function' => 'dropboxAuthorize'));
			exit;
		}

		$this->Session->write('general_msg', __('Service Saved!','pressbackup'));
		$this->redirect($service['dashboard']);
	}

	/**
	 * Show the form to modify prefered settings
	 * as backup type, schedule time, etc
	 */
	public function configSettings()
	{
		$settings = get_option('pressbackup.preferences');

		if ($this->Session->check('error')) {
			$this->Msg->set('error',$this->Session->read('error_msg'));
			$this->Session->delete('error');
			$this->Session->delete('error_msg');
		}

		$this->set('settings', get_option('pressbackup.preferences'));
		$this->set('service', $this->Misc->currentService());
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	public function configSettingsSave()
	{
		//exit if no data or invalid
		if (!isset($_POST['data']) || !$this->checkFieldsStep2($_POST['data']['preferences'])) {
			$this->Session->write('error', true);
			$this->redirect(array('controller' => 'settings', 'function' => 'configSettings'));
		}

		$settings = get_option('pressbackup.preferences');

		$this->import('Scheduler.php');
		$sch = new SchedulerPBLib();


		//saving number of copies to store
		if (isset($_POST['data']['preferences']['copies'])) {
			$settings['backup']['copies'] = $_POST['data']['preferences']['copies'];
		} else {
			$settings['backup']['copies'] = 5;
		}

		//Check if davanced options were activated
		$advanced_activated = $_POST['data']['preferences']['backup_advanced']['activated'];

		// saving backup schedule time
		if ($advanced_activated) {
			$scheduled = $sch->minScheduledTaskTime($_POST['data']['preferences']['backup_advanced']);
			$settings['backup']['time'] = $scheduled['time'];
		}
		elseif ( isset ($_POST['data']['preferences']['time']) ) {
			$settings['backup']['time'] = $_POST['data']['preferences']['time'];
		}
		else {
			$settings['backup']['time'] = 0;
		}

		//setting backup type
		if($advanced_activated){
			$curdate = strtotime($this->Misc->midnightUTC());
			$post = $_POST['data']['preferences']['backup_advanced'];

			$settings['backup_advanced']['activated'] = true;
			$settings['backup_advanced']['db']= array('time' => $post['db']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['plugins']= array('time' => $post['plugins']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['themes']= array('time' => $post['themes']['time'], 'last_date' => $curdate);
			$settings['backup_advanced']['uploads']= array('time' => $post['uploads']['time'], 'last_date' => $curdate);

			$settings['backup']['type'] = $sch->activatedTasks($settings['backup_advanced']);
		}else{
			$settings['backup_advanced']['activated'] = false;
			$settings['backup']['type'] = join(',', $_POST['data']['preferences']['type']);
		}

		update_option('pressbackup.preferences', $settings);

		$service = $this->Misc->currentService();
		if ($service['id'] != 'none') {
			$sch->add();
		}

		$this->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$this->redirect($service['dashboard']);
	}


	/**
	 * Show the form to modify prefered conpatibility settings
	 */
	public function configCompatibility()
	{
		$settings = get_option('pressbackup.preferences');

		$binaries = false;
		if ($this->Misc->checkShell('php')) {
			$binaries = true;
		}

		$this->set('binaries', $binaries);
		$this->set('settings', $settings);
	}

	/**
	 * Store the modified settings if they are correct
	 * or save the error and return to form page
	 */
	public function configCompatibilitySave()
	{
		$settings = get_option('pressbackup.preferences');

		if (isset($_POST['data']['compatibility']['background'])) {
			$settings['compatibility']['background'] = $_POST['data']['compatibility']['background'];
		}

		if (isset($_POST['data']['compatibility']['zip'])) {
			$settings['compatibility']['zip'] = $_POST['data']['compatibility']['zip'];
		}

		update_option('pressbackup.preferences', $settings);

		$this->Session->write('general_msg', __('Preferences Saved!','pressbackup'));
		$this->redirect(array('controller' => 'main', 'function' => 'dashboard'));
	}


	/**
	 * Store the settings if they are correct
	 * or save the error and return to form page
	 *
	 * @token_s string: token generated on revios step
	 */
	public function dropboxAuthorize($token_secret)
	{
		$settings = get_option('pressbackup.preferences');
		$srv = $this->Misc->serviceObject('dBox', array(null, null));

		//get real tokens
		$token = $_GET['oauth_token'];
		$permanentTokens = $srv->getAccessToken($token, $token_secret);

		//store tokens
		$settings['Service']['dropbox']['credential'] = base64_encode($permanentTokens['oauth_token'] . '|AllYouNeedIsLove|' . $permanentTokens['oauth_token_secret']);
		update_option('pressbackup.preferences', $settings);

		if($settings['configured']){
			$this->Session->write('general_msg', __('Service Saved!','pressbackup'));
			$this->redirect(array('controller' => 'main', 'function' => 'dashboard'));
		} else {
			$this->redirect(array('controller' => 'settings', 'function' => 'configStep2'));
		}
	}

	/**
	 * Check if credentials are valid
	 * for S3 or Pressbackup PRO account
	 *
	 * @args array: credentials sent via post
	 */
	public function checkFieldsStep1($args)
	{
		if ($args['service'] == 's3') {
			$PS3 = $args['preferences']['S3'];

			//check empty data
			if (!isset($PS3['accessKey']) || empty($PS3['accessKey']) || !isset($PS3['secretKey']) || empty($PS3['secretKey'])) {
				$this->Session->write('error_msg', __('Empty Fields, please ty it again','pressbackup'));
				return false;
			}

			$srv = $this->Misc->serviceObject('S3', array($PS3['accessKey'], $PS3['secretKey']));
			$buckets = $srv->listBuckets();

			//check if credentials are right
			if (!$buckets) {
				$this->Session->write('error_msg', __('Incorrect S3 access or secret Keys','pressbackup'));
				return false;
			}
			return true;
		} 

		if ($_POST['data']['service'] == 'pro') {

			$PPRO = $args['preferences']['pressbackuppro'];

			$srv = $this->Misc->serviceObject('Pro', array(null, null));

			if(!$res = $srv->auth()) {
				$this->Session->write('error_msg', __('Please register your site in https://pressbackup.com', 'pressback'));
				return false;
			}

			if($res['package'] == 'pro'){
				$index = 'pressbackuppro';
			}else{
				$index = 'pressexpress';
			}

			$_POST['data']['service'] = $res['package'];
			$_POST['data']['preferences'][$index]['user'] = $res['username'];
			$_POST['data']['preferences'][$index]['key'] = $res['authkey'];

			return true;
		} 

		if ($args['service'] == 'local') {

			$PLOCAL = $args['preferences']['local'];

			$srv = $this->Misc->serviceObject('Local', null);


			if (!$srv->checkExists($PLOCAL['path'])) {
				$this->Session->write('error_msg', __("The specified folder does not exist.",'pressbackup'));
				return false;
			}

			if (!$srv->checkPermissions($PLOCAL['path'])) {
				$this->Session->write('error_msg', __("No write permissions for the selected folder",'pressbackup'));
				return false;
			}

			return true;
		}

		return true;
	}

	/**
	 * Check if fields for Settings are valid.
	 * The files are type of backup, schedule time, etc
	 *
	 * @args array: all fields sent via post
	 */
	public function checkFieldsStep2($args)
	{
		if(!isset($args['backup_advanced']['activated']) ){

			if (isset($args['time']) && !preg_match("/^[[:digit:]]+$/", $args['time'])) {
				$this->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
				return false;
			}

			if (isset($args['copies']) && !preg_match("/^[[:digit:]]+$/", $args['copies'])) {
				$this->Session->write('error_msg', __('Specify the number of backups to be stored','pressbackup'));
				return false;
			}

			if (isset($args['type']) && empty($args['type'])) {
				$this->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}

			for ($i = 0; $i < count($args['type']); $i++) {
				if (!preg_match("/^[[:digit:]]+$/", $args['type'][$i])) {
					$this->Session->write('error_msg', __('Empty/wrong Fields, please ty it again','pressbackup'));
					return false;
				}
			}

		}else {
			unset($args['backup_advanced']['activated']); 
			$at_least_one = false;

			//check that all times received they are digit, and that at least one is activated,
			foreach($args['backup_advanced'] as $type=>$data){
				if (isset($data['time']) && !preg_match("/^[[:digit:]]+$/", $data['time'])) {
					$this->Session->write('error_msg', __('Specify the scheduled backup time range','pressbackup'));
					return false;
				}
				if($data['time'] != 0){
					$at_least_one = true;
				}
			}

			if(!$at_least_one){
				$this->Session->write('error_msg', __('Specify what you would like to backup','pressbackup'));
				return false;
			}
		}
		return true;
	}

	//cron checks

	public function wizardStartCronTask ()
	{
		update_option('pressbackup.wizard.cronTaskStatus', 'success');
	}

	public function wizardChkCronTaskStatus ()
	{
		$status = get_option('pressbackup.wizard.cronTaskStatus');

		$response = array('status'=> 'fail');
		if ($status == 'success') {
			$response = array('status'=> 'ok');
		}

		//clean output buffer
		ob_end_clean();

		//send json
		header("Status: 200", true);
		header("Content-type: application/json", true);
		echo json_encode($response);
		exit;
	}

	//when wizardChkCronTaskStatus ajax response is X times 'fail'
	//this function must be called to set the status of the process
	//to fail
	public function checkCronFail ()
	{
		update_option('pressbackup.wizard.cronTaskStatus', 'fail');
		$this->redirect(array('controller'=>'settings', 'function'=>'configInit'));
	}
	

}
?>
