<?php

/**
* Backup Functions lib for Pressbackup.
*
* This Class provide the functionality to mannage backups
* functions for creation, restoring, get and save from S3 and Ppro
*
* Licensed under The GPL v2 License
* Redistributions of files must retain the above copyright notice.
*
* @link			http://pressbackup.com
* @package		libs
* @subpackage	libs.backups
* @since		0.1
* @license		GPL v2 License
*/

class BackupsManagerPBLib
{
	/**
	 * Local Reference to the framework core object
	 * 
	 * @var object
	 * @access private
	 */
	private $fp = null;

	/**
	 * Local Copy to Session Component
	 * 
	 * @var object
	 * @access public
	 */
	public $Session = null;

	/**
	 * Local Copy to Misc Lib
	 * 
	 * @var object
	 * @access public
	 */
	public $Misc = null;

	/**
	 * Constructor.
	 *
	 * @param object FramePress Core
	 * @access public
	 */
	public function __construct ()
	{
		global $pressbackup;
		$this->fp = &$pressbackup;

		$pressbackup->import('Misc.php');
		$this->Misc = new MiscPBLib();

		$pressbackup->import('session.php');
		$this->Session = new PressbackupSession( $pressbackup );
	}

	//----------------------------------------------------------------------------------------

	/**
	 * Create a Backup file of the system
	 * 
	 * Create a backup file with the content specified on settings
	 */
	public function create ()
	{
		//set infinite time for this action
		set_time_limit (0);

		//tools and info
		$settings=get_option('pressbackup.preferences');

		//clean log && tmp dir
		$this->Misc->cleanTmpFordes();

		//write log file so foreground process can know what we are doing
		//begining!
		$this->Misc->writeLogFile('create.log', 'start');

		//zip files and export db
		if(!$this->backupFiles() || !$this->backupDB())
		{
			return false;
		}

		//Zip container: file name 
		$backup_file_type = str_replace(',', '-', $settings['backup']['type']);
		$blog_name = str_replace(' ', '-', $this->Misc->normalizeString(strtolower(get_bloginfo( 'name' ))));
		$zip_file_name = $blog_name.'-backup_'.$backup_file_type.'_'.date('M-d-His').'.zip';

		//what type of zip creation must we use?
		$type = 'shell';
		if( $settings['compatibility']['zip'] == 10 ) { $type = 'php'; }

		//Create xip container
		$res = $this->Misc->zip($type, array(
			'context_dir'=>$this->fp->Path->Dir['SYSTMP'],
			'dir' => basename( $this->fp->Path->Dir['PBKTMP']) . DS,
			'zip' => $this->fp->Path->Dir['PBKTMP'] . DS . $zip_file_name,
			'compression' => 0
		));

		//check response of zip creation
		if( !$res ) {
			//inform error
			$this->Session->write( 'error_msg', __("Zip file is corrupt - creation failed",'pressbackup'), true);

			//write log file so foreground process can know what we are doing
			//something went wrong
			$this->Misc->writeLogFile('create.log', 'fail');

			return false;
		}

		//write log file so foreground process can know what we are doing
		//all was OK!
		$this->Misc->writeLogFile('create.log', 'finish');
		return $this->fp->Path->Dir['PBKTMP'] . DS . $zip_file_name;
	}

	/**
	 * Create a Backup file of wp-content folders
	 */
	private function backupFiles()
	{
		//tools and info
		$settings=get_option('pressbackup.preferences');

		//read what to backup
		$backup_file_type = explode(',', $settings['backup']['type']);

		//uploads
		if(in_array('1', $backup_file_type) && file_exists(WP_CONTENT_DIR. DS.'uploads'))
		{

			//write log file so foreground process can know what we are doing
			//begining upload backup!
			$this->Misc->writeLogFile('create.log', __('Creating Uploads folder backup','pressbackup'));

			//what type of zip creation must we use?
			$type = 'shell';
			if( $settings['compatibility']['zip'] == 10 ) { $type = 'php'; }

			//Create uploads zip container
			$res = $this->Misc->zip($type, array(
				'context_dir' => WP_CONTENT_DIR,
				'dir' => 'uploads'.DS,
				'zip' => $this->fp->Path->Dir['PBKTMP'].DS.'uploads.zip'
			));

			//check response of zip creation
			if( !$res ) {
				//write log file so foreground process can know what we are doing
				//something went wrong
				$this->Misc->writeLogFile('create.log', 'fail');
				return false;
			}
		}

		//plugins
		if(in_array('3', $backup_file_type))
		{
			//write log file so foreground process can know what we are doing
			//begining plugins backup!
			$this->Misc->writeLogFile('create.log', __('Creating Plugins folder backup','pressbackup'));

			//what type of zip creation must we use?
			$type = 'shell';
			if( $settings['compatibility']['zip'] == 10 ) { $type = 'php'; }

			//Create uploads zip container
			$res = $this->Misc->zip($type, array(
				'context_dir' => WP_CONTENT_DIR,
				'dir' => 'plugins'.DS,
				'zip' => $this->fp->Path->Dir['PBKTMP'].DS.'plugins.zip'
			));

			//check response of zip creation
			if( !$res ) {
				//write log file so foreground process can know what we are doing
				//something went wrong
				$this->Misc->writeLogFile('create.log', 'fail');
				return false;
			}
		}

		//themes
		if(in_array('5', $backup_file_type))
		{
			//write log file so foreground process can know what we are doing
			//begining themes backup!
			$this->Misc->writeLogFile('create.log', __('Creating Themes folder backup','pressbackup'));

			//what type of zip creation must we use?
			$type = 'shell';
			if( $settings['compatibility']['zip'] == 10 ) { $type = 'php'; }

			//Create uploads zip container
			$res = $this->Misc->zip($type, array(
				'context_dir' => WP_CONTENT_DIR,
				'dir' => 'themes'.DS,
				'zip' => $this->fp->Path->Dir['PBKTMP'].DS.'themes.zip'
			));

			//check response of zip creation
			if( !$res ) {
				//write log file so foreground process can know what we are doing
				//something went wrong
				$this->Misc->writeLogFile('create.log', 'fail');
				return false;
			}
		}
		return true;
	}

	/**
	 * Create a Backup file with the SQL dump
	 */
	private function backupDB ()
	{
		//tools and info
		global $wpdb;
		$settings=get_option('pressbackup.preferences');

		//maximun multimple inserts
		$insert_max = 50;

		//read backup preferences
		$backup_db_type = explode(',', $settings['backup']['type']);

		//Return if no DB backu needed 
		if(!in_array('7', $backup_db_type)) { return true; }

		//write log file so foreground process can know what we are doing
		//begining themes backup!
		$this->Misc->writeLogFile('create.log', __('Creating Database backup','pressbackup'));

		//save .httaccess for this server
		try { @copy(ABSPATH.'.htaccess', $this->fp->Path->Dir['PBKTMP'].DS.'.htaccess'); } catch (Exception $e) { }

		//save server for this SQL
		$file =$this->fp->Path->Dir['PBKTMP'].DS.'server';
		if(!$fh=fopen( $file, 'w'))
		{
			$this->Session->write( 'error_msg',  __("can not create file to store server for this SQL dump",'pressbackup'), true);
			$this->Misc->writeLogFile('create.log', 'fail');
			return false;
		}
		fwrite($fh, get_bloginfo( 'wpurl' ));
		fclose($fh);

		//create .sql file
		$file = $this->fp->Path->Dir['PBKTMP'].DS.'database.sql';
		if(!$fh=fopen( $file, 'w'))
		{
			$this->Session->write( 'error_msg',  __("can not create file for SQL dump",'pressbackup'), true);
			$this->Misc->writeLogFile('create.log', 'fail');
			return false;
		}

		//dump DB
		$file_header= '-- PressBackup SQL Dump'."\n".
		'-- version 1.0'."\n".
		'-- http://www.infinimedia.com'."\n\n".
		'SET SQL_MODE="NO_AUTO_VALUE_ON_ZERO";'."\n\n";
		fwrite($fh, $file_header."\n\n");

		$DB_tables=$wpdb->get_results('SHOW TABLES');
		$method2 = 'Tables_in_'.DB_NAME;

		for($i=0; $i<count($DB_tables); $i++)
		{

			//table estructure
			$query = $wpdb->get_results('show create table '.$DB_tables[$i]->$method2);

			//"Create Table" with a space in the middle.
			$method = 'Create Table';
			$table_structure = $query[0]->$method;
			$table_structure = str_replace ('CREATE TABLE', 'CREATE TABLE IF NOT EXISTS', $table_structure);
			fwrite($fh, "\n\n".$table_structure.";\n\n");

			//inserts -header
			$describe_table = $wpdb->get_results('DESCRIBE '.$DB_tables[$i]->$method2);
			$insert_header=$fields=array();
			for ($j=0; $j< count($describe_table); $j++)
			{
				$insert_header[$j] = '`'. $describe_table[$j]->Field.'`';
				$fields[$j]=$describe_table[$j]->Field;
			}
			$insert_header= 'INSERT INTO `'.$DB_tables[$i]->$method2.'` ( '. join(',', $insert_header). ') VALUES ';


			//count rows for inserts -data
			$inserts_count = $wpdb->get_results('SELECT count(*) as cant FROM  `'.$DB_tables[$i]->$method2.'`');
			$insert_pages = ceil ($inserts_count[0]->cant / $insert_max);

			//dump insert rows
			for ($l = 0; $l < $insert_pages; $l ++)
			{
				//inserts -data
				unset($inserts); $inserts = array();
				$inserts = $wpdb->get_results('SELECT * FROM  `'.$DB_tables[$i]->$method2.'` LIMIT '.($l * $insert_max).','.$insert_max);

				unset($insert_data); $insert_data=array();
				for ($j=0; $j< count($inserts); $j++)
				{
					unset($popo); $popo=array();
					for ($k=0; $k< count($fields); $k++)
					{
						$popo[$k]='\''.mysql_real_escape_string($inserts[$j]->$fields[$k]).'\'';
					}
					$insert_data[$j] = '('.join(',', $popo).')';
				}
				if($insert_data) {
					fwrite($fh, "\n".$insert_header."\n".join(",\n", $insert_data).';'."\n\n");
				}
			}
		}
		fclose($fh);

		return true;
	}

	//----------------------------------------------------------------------------------------

	/**
	 * Save a File on the correct storage service
	 * 
	 * This is a short cut for save on * functions
	 */
	public function save ($zip_file)
	{
		set_time_limit (0);

		//write log file so foreground process can know what we are doing
		//begining backup storage!
		$this->Misc->writeLogFile('sent.log', 'start');

		$service = $this->Misc->currentService();

		if ($service['id'] == 'S3'){
			return $this->saveOnS3($zip_file);
		}

		elseif ($service['id'] == 'Pro'){
			return $this->saveOnPRO($zip_file);
		}

		elseif ($service['id'] == 'Exp'){
			return $this->saveOnPRO($zip_file);
		}

		elseif ($service['id'] == 'dBox'){
			return $this->saveOnDropbox($zip_file);
		}

		elseif ($service['id'] == 'Local'){
			return $this->saveOnLocal($zip_file);
		}
	}


	private function saveOnS3($zip_file)
	{
		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//fix for no occidental characers, basename and pathinfo remove them from $zip_file
		$pathinfo = explode(DS, $zip_file);
		$name = array_pop($pathinfo);

		//send file
		if(!$srv->putObjectFile($zip_file, $service['bucket_name'], $name, ServiceS3PBLib::ACL_PRIVATE))
		{
			$this->Session->write( 'error_msg',  __("can not save file on S3",'pressbackup'), true);
			$this->Misc->writeLogFile('sent.log', 'fail');
			return false;
		}

		//check the number of backup stored
		$bucket_files = $srv->getBucket( $service['bucket_name']);
		$this->removeExcessBackups($service, $bucket_files);

		//save massage
		$this->Session->write( 'general_msg', __('Backup saved!','pressbackup'), true);
		$this->Misc->writeLogFile('sent.log', 'finish');
		return true;
	}

	private function saveOnPRO($zip_file)
	{
		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//send file
		if(!$srv->putFile($zip_file)) {

			$message = __("Connection with Pressbackup Pro fail. Try again later",'pressbackup');
			if ($srv->response['body'] && $aux = @json_decode($srv->response['body'], true)) {
				$message = $aux['msg'];
			} 

			$this->Session->write( 'error_msg', $message, true);
			$this->Misc->writeLogFile('sent.log', 'fail');
			return false;
		}

		//check the number of backup stored
		$bucket_files = $srv->getFilesList();
		$this->removeExcessBackups($service, $bucket_files);

		//save massage
		$this->Session->write( 'general_msg', __('Backup saved!','pressbackup'), true);
		$this->Misc->writeLogFile('sent.log', 'finish');
		return true;
	}

	private function saveOnDropbox($zip_file)
	{
		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//send file
		if(!$srv->putFile($zip_file)) {
			$this->Session->write( 'error_msg',  __("Connection with Dropbox fail. Try again later",'pressbackup'), true);
			$this->Misc->writeLogFile('sent.log', 'fail');
			return false;
		}

		//check the number of backup stored
		$bucket_files = $srv->getFilesList();
		$this->removeExcessBackups($service, $bucket_files);

		//save massage
		$this->Session->write( 'general_msg', __('Backup saved!','pressbackup'), true);
		$this->Misc->writeLogFile('sent.log', 'finish');
		return true;
	}

	private function saveOnLocal($zip_file)
	{
		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//send file
		if(!$srv->putFile($zip_file)) {
			$this->Session->write( 'error_msg',  __("Can not store the file in the local host",'pressbackup'), true);
			$this->Misc->writeLogFile('sent.log', 'fail');
			return false;
		}

		//check the number of backup stored
		$bucket_files = $srv->getFilesList();
		$this->removeExcessBackups($service, $bucket_files);

		//save massage
		$this->Session->write( 'general_msg', __('Backup saved!','pressbackup'), true);
		$this->Misc->writeLogFile('sent.log', 'finish');
		return true;
	}


	private function removeExcessBackups ($service, $backups)
	{
		//settings
		$settings=get_option('pressbackup.preferences');

		$backups = @$this->Misc->msort($service['id'], $backups);
		$backups = $this->Misc->filterFiles('this_site', $backups);
		if($settings['backup']['copies'] != 7 && count($backups) > $settings['backup']['copies']) {
			$file =  array_pop($backups);
			$this->delete($file['name'], false);
		}
	}

	//----------------------------------------------------------------------------------------

	public function restore ($args)
	{
		//set infinite time for this action
		set_time_limit (0);

		//clean log && tmp dir
		$this->Misc->cleanTmpFordes();

		//try to open the zip file
		$zip = new ZipArchive();
		if(!$zip->open($args['tmp_name'])===true) {
			$this->Session->write( 'error_msg',  __("Backup seems corrupt. Process aborted",'pressbackup'));
			return false;
		}

		//shortcuts
		$PBKTMP = $this->fp->Path->Dir['PBKTMP'];
		$SYSTMP = $this->fp->Path->Dir['SYSTMP'];

		//on a migration or blog name change tmp folder change its name
		$oldTmpFolderName = dirname ($zip->getNameIndex(0));
		$newTmpFolderName = basename($PBKTMP);

		//this will use/create the tmp folder with the old name
		$zip->extractTo($SYSTMP );
		$zip->close();

		//rename old folder if necesary
		if( $oldTmpFolderName != $newTmpFolderName ){
			@rmdir ($SYSTMP . DS . $newTmpFolderName);
			@rename ( $oldTmpFolderName , $newTmpFolderName );
		}

		//restore backup
		if(!$this->restoreFiles() || !$this->restoreDB())
		{
			return false;
		}

		$this->Session->write( 'general_msg',  __("System restored!",'pressbackup'));
		return true;
	}

	private function restoreFiles()
	{

		//shortcuts
		$PBKTMP = $this->fp->Path->Dir['PBKTMP'].DS;

		$zip = new ZipArchive();

		//restore themes
		if(file_exists($PBKTMP .'themes.zip')){

			//check file healt
			if($zip->open( $PBKTMP .'themes.zip') !== true) {
				$this->Session->write( 'error_msg',  __("Can not restore themes, themes backup is corrupt. Restore process aborted",'pressbackup'));
				return false;
			}

			//try to extract it
			if(!@$zip->extractTo(WP_CONTENT_DIR)){
				$zip->close();
				$this->Session->write( 'error_msg',  __("Can not restore themes folder, permission denied to write on <b>wp-content</b> foder. Restore process aborted",'pressbackup'));
				return false;
			}
			$zip->close();
		}

		//restore uploads
		if(file_exists($PBKTMP .'uploads.zip')){

			//check file healt
			if($zip->open( $PBKTMP .'uploads.zip') !== true) {
				$this->Session->write( 'error_msg',  __("Can not restore uploads folder, uploads backup is corrupt. Restore process aborted",'pressbackup'));
				return false;
			}

			//try to extract it
			if(!@$zip->extractTo(WP_CONTENT_DIR))
			{
				$zip->close();
				$this->Session->write( 'error_msg',  __("Can not restore uploads folder, permission denied to write on <b>wp-content</b> foder. Restore process aborted",'pressbackup'));
				return false;
			}

			$zip->close();
		}

		//restore plugins
		if(file_exists($PBKTMP .'plugins.zip')){

			//check file healt
			if($zip->open( $PBKTMP .'plugins.zip') !== true) {
				$this->Session->write( 'error_msg',  __("Can not restore plugins folder, plugins backup is corrupt. Restore process aborted",'pressbackup'));
				return false;
			}

			//try to extract it
			if(!@$zip->extractTo(WP_CONTENT_DIR)){
				$zip->close();
				$this->Session->write( 'error_msg',  __("Can not restore plugins folder, permission denied to write on <b>wp-content</b> foder. Restore process aborted",'pressbackup'));
				return false;
			}
			$zip->close();
		}

		$this->Misc->actionFolder(WP_CONTENT_DIR , array('function' => 'chmod', 'param' => array('file' =>0644, 'dir'=>0755)));

		return true;
	}

	private function restoreDB()
	{
		require_once(ABSPATH . '/wp-admin/admin.php');
		require_once( ABSPATH . '/wp-admin/includes/upgrade.php' );

		//tools and info
		global $wpdb;
		global $wp_rewrite;

		//shortcuts
		$PBKTMP = $this->fp->Path->Dir['PBKTMP'].DS;

		//check if need to restore DB
		if(!file_exists($PBKTMP .'database.sql')){ return true; }

		//get old server name to can restore DB with it
		if(!$fn=fopen( $PBKTMP .'server', 'rb')) {
			$this->Session->write( 'error_msg',  __("Can not restore database, missing files. Restore process aborted",'pressbackup'));
			return false;
		}

		//for migrations
		$last_server=trim(fgets($fn)); fclose($fn);
		$new_server = get_bloginfo( 'wpurl' );

		//get SQL dump
		if(!$DBdump = @fopen($PBKTMP .'database.sql', 'rb')) {
			$this->Session->write( 'error_msg',  __("Can not restore database, missing files (.sql). Restore process aborted",'pressbackup'));
			return false;
		}

		//Drop current DB tables
		$DB_tables=$wpdb->get_results('SHOW TABLES');
		$method = 'Tables_in_'.DB_NAME;
		for($i=0; $i<count($DB_tables); $i++) {

			$wpdb->query('DROP TABLE '.$DB_tables[$i]->$method);
		}

		//Read headers from .sql
		 $inserts='';
		while (($buffer=fgets($DBdump)) && !preg_match("/^INSERT INTO(.)*/", $buffer) && !preg_match("/^CREATE TABLE IF NOT EXISTS(.)*/", $buffer))
		{
			$inserts .= $buffer;
		}
		$inserts = str_replace($last_server, $new_server, $inserts); //echo $inserts;
		@$wpdb->query(trim($inserts));

		//Read until EOF cuting sql into create and insert stataments
		while($buffer)
		{
			if(preg_match("/^CREATE TABLE IF NOT EXISTS(.)*/", $buffer)){

				$inserts = $buffer;
				while(($buffer=fgets($DBdump)) && !preg_match("/^INSERT INTO(.)*/", $buffer) && !preg_match("/^CREATE TABLE IF NOT EXISTS(.)*/", $buffer))
				{
					$inserts .= $buffer;
				}
				$inserts = str_replace($last_server, $new_server, $inserts); //echo $inserts;
				dbDelta( trim($inserts) ); //@$wpdb->query(trim($inserts));
			}

			if(preg_match("/^INSERT INTO(.)*/", $buffer)) {

				//save insert header
				$header = $buffer;

				// start read insert values
				$i=0; $inserts='';
				while(($buffer=fgets($DBdump)) && !preg_match("/^INSERT INTO(.)*/", $buffer) && !preg_match("/^CREATE TABLE IF NOT EXISTS(.)*/", $buffer))
				{
					$inserts .= $buffer; $i++;

					//inser 50 results at once
					if($i==50) {
						$inserts= trim($inserts);
						if( substr($inserts, -1) == ',') { $inserts = substr_replace($inserts, ';', -1, 1);}
						$inserts = str_replace($last_server, $new_server, $inserts);
						@$wpdb->query(trim($header.$inserts)); //echo $header.$inserts;
						$i=0; $inserts='';
					}
				}

				//if something remaing to save
				if($i>0) {
					$inserts= trim($inserts);
					if( substr($inserts, -1) == ',') { $inserts = substr_replace($inserts, ';', -1, 1);}
					$inserts = str_replace($last_server, $new_server, $inserts);
					@$wpdb->query(trim($header.$inserts)); //echo trim($header.$inserts)."\n\n";

				}
			}

		}
		fclose($DBdump);

		//update options
		$siteopts = wp_load_alloptions();
		$this->updateSiteOptions($siteopts, $last_server, $new_server);

		//copy backed up .htaccess
		try { @copy($PBKTMP .'.htaccess', ABSPATH .'.htaccess') ; } catch (Exception $e) {} 

		//remake .htaccess
		//see extra controller for this part
		$settings= get_option('pressbackup.preferences');
		$settings['restore']=true;
		update_option('pressbackup.preferences',$settings);

		return true;
	}

	private function updateSiteOptions(array $options, $old_url, $new_url)
	{
		require_once ABSPATH .'wp-includes/functions.php';
		foreach ($options as $option_name => $option_value) {

			if (FALSE === strpos($option_value, $old_url)) {
				continue;
			}

			if (is_array($option_value)) {
				$this->updateSiteOptions($option_value, $old_url, $new_url);
			}

			// attempt to unserialize option_value
			if(!is_serialized($option_value)) {
				$newvalue = str_replace($old_url, $new_url, $option_value);
			} else {
				$newvalue = $this->updateSerializedOptions(maybe_unserialize($option_value), $old_url, $new_url);
			}

			update_option($option_name, $newvalue);
		}
	}

	private function updateSerializedOptions($data, $old_url, $new_url) {
		require_once ABSPATH .'wp-includes/functions.php';
		// ignore _site_transient_update_*
		if(is_object($data)){
			return $data;
		}

		foreach ($data as $key => $val) {
			if (is_array($val)) {
					$data[$key] = $this->updateSerializedOptions($val, $old_url, $new_url);
			} else {
				if (!strstr($val, $old_url)) {
					continue;
				}
				$data[$key] = str_replace($old_url, $new_url, $val);
			}
		}
		return $data;
	}

	//----------------------------------------------------------------------------------------

	public function get ($file='')
	{
		//set infinite time for this action
		@set_time_limit (0);

		if(!$file){
			$this->Session->write( 'error_msg',  __("Missing backup file name",'pressbackup'));
			return false;
		}

		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//where to put backup file
		$stored_in = $store_in = $this->fp->Path->Dir['SYSTMP']. DS . $file;

		// succesfully transfer ?
		$transfer = false;

		//retrive backups from Amazon S3
		if( $service['id'] == 'S3' ){
			$transfer = $srv->getObject($service['bucket_name'], $file, $store_in);
		}

		//retrive backups from Pro
		elseif( $service['id'] == 'Pro' ||  $service['id'] == 'Exp' ) {
			$transfer = $srv->getFile($file, $store_in);
		}

		//retrive backups from Dropbox
		elseif( $service['id'] == 'dBox' ) {
			$transfer = $srv->getFile($file, $store_in);
		}

		//retrive backups from Local
		elseif( $service['id'] == 'Local' ) {
			$transfer = $srv->getFile($file, $store_in);
		}


		if(!$transfer){
			$this->Session->write( 'error_msg',  __('Failed to get file','pressbackup'));
			return false;
		}

		return $stored_in;
	}

	/* Only for pressbackup pro*/
	public function get2 ($file='')
	{
		//set infinite time for this action
		set_time_limit (0);

		if(!$file){
			$this->Session->write( 'error_msg',  __("Missing backup file name",'pressbackup'));
			return false;
		}

		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//succesfully transfer ?
		$transfer = false;

		if( $service['id'] == 'Pro' || $service['id'] == 'Exp' ) {
			$transfer = $srv->getFile2($file);
		}

		if(!$transfer) {
			$this->Session->write( 'error_msg',  __('Failed to get file 2','pressbackup'));
			return false;
		} 

		return $transfer;
	}

	//----------------------------------------------------------------------------------------

	public function delete($file='', $inform=true)
	{
		//set infinite time for this action
		set_time_limit (0);

		if(!$file){
			if($inform){ $this->Session->write( 'error_msg',  __("Missing backup file name",'pressbackup')); }
			return false;
		}

		//deleted ?
		$deleted = false;

		//get Service object && settings
		$service = $this->Misc->currentService();
		$srv = $this->Misc->serviceObject($service['id']);

		//Delete backups from Amazon S3
		if( $service['id'] == 'S3' ) {
			$deleted = $srv->deleteObject($service['bucket_name'],$file);
		}

		//Delete backups from Pro
		elseif( $service['id'] == 'Pro' || $service['id'] == 'Exp') {
			$deleted = $srv->deleteFile($file);
		}

		//Delete backups from Pro
		elseif( $service['id'] == 'dBox' ) {
			$deleted = $srv->deleteFile($file);
		}

		//Delete backup from localhost
		elseif( $service['id'] == 'Local' ) {
			$deleted = $srv->deleteFile($file);
		}

		if(!$deleted) {
			return false;
		}

		if($inform){ $this->Session->write( 'general_msg',  __('Backup deleted!','pressbackup')); }
		return true;
	}

}
?>
